

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

The class of a flags type holds information about its
possible values.
-}

module GI.GObject.Structs.FlagsClass
    ( 

-- * Exported types
    FlagsClass(..)                          ,
    newZeroFlagsClass                       ,
    noFlagsClass                            ,


 -- * Properties
-- ** GTypeClass
    flagsClassReadGTypeClass                ,


-- ** Mask
    flagsClassReadMask                      ,


-- ** NValues
    flagsClassReadNValues                   ,


-- ** Values
    flagsClassReadValues                    ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.GObject.Types
import GI.GObject.Callbacks

newtype FlagsClass = FlagsClass (ForeignPtr FlagsClass)
-- | Construct a `FlagsClass` struct initialized to zero.
newZeroFlagsClass :: MonadIO m => m FlagsClass
newZeroFlagsClass = liftIO $ callocBytes 24 >>= wrapPtr FlagsClass

noFlagsClass :: Maybe FlagsClass
noFlagsClass = Nothing

flagsClassReadGTypeClass :: FlagsClass -> IO TypeClass
flagsClassReadGTypeClass s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO (Ptr TypeClass)
    val' <- (newPtr 8 TypeClass) val
    return val'

flagsClassReadMask :: FlagsClass -> IO Word32
flagsClassReadMask s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO Word32
    return val

flagsClassReadNValues :: FlagsClass -> IO Word32
flagsClassReadNValues s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 12) :: IO Word32
    return val

flagsClassReadValues :: FlagsClass -> IO FlagsValue
flagsClassReadValues s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO (Ptr FlagsValue)
    val' <- (newPtr 24 FlagsValue) val
    return val'

type family ResolveFlagsClassMethod (t :: Symbol) (o :: *) :: * where
    ResolveFlagsClassMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveFlagsClassMethod t FlagsClass, MethodInfo info FlagsClass p) => IsLabelProxy t (FlagsClass -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveFlagsClassMethod t FlagsClass, MethodInfo info FlagsClass p) => IsLabel t (FlagsClass -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


