

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

A structure that provides information to the type system which is
used specifically for managing interface types.
-}

module GI.GObject.Structs.InterfaceInfo
    ( 

-- * Exported types
    InterfaceInfo(..)                       ,
    newZeroInterfaceInfo                    ,
    noInterfaceInfo                         ,


 -- * Properties
-- ** InterfaceData
    interfaceInfoReadInterfaceData          ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.GObject.Types
import GI.GObject.Callbacks

newtype InterfaceInfo = InterfaceInfo (ForeignPtr InterfaceInfo)
-- | Construct a `InterfaceInfo` struct initialized to zero.
newZeroInterfaceInfo :: MonadIO m => m InterfaceInfo
newZeroInterfaceInfo = liftIO $ callocBytes 24 >>= wrapPtr InterfaceInfo

noInterfaceInfo :: Maybe InterfaceInfo
noInterfaceInfo = Nothing

-- XXX Skipped getter for "InterfaceInfo:interface_init" :: Not implemented: "Wrapping foreign callbacks is not supported yet"

-- XXX Skipped getter for "InterfaceInfo:interface_finalize" :: Not implemented: "Wrapping foreign callbacks is not supported yet"

interfaceInfoReadInterfaceData :: InterfaceInfo -> IO (Ptr ())
interfaceInfoReadInterfaceData s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO (Ptr ())
    return val

type family ResolveInterfaceInfoMethod (t :: Symbol) (o :: *) :: * where
    ResolveInterfaceInfoMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveInterfaceInfoMethod t InterfaceInfo, MethodInfo info InterfaceInfo p) => IsLabelProxy t (InterfaceInfo -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveInterfaceInfoMethod t InterfaceInfo, MethodInfo info InterfaceInfo p) => IsLabel t (InterfaceInfo -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


