

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

A structure holding in-depth information for a specific signal. It is
filled in by the g_signal_query() function.
-}

module GI.GObject.Structs.SignalQuery
    ( 

-- * Exported types
    SignalQuery(..)                         ,
    newZeroSignalQuery                      ,
    noSignalQuery                           ,


 -- * Properties
-- ** Itype
    signalQueryReadItype                    ,


-- ** NParams
    signalQueryReadNParams                  ,


-- ** ReturnType
    signalQueryReadReturnType               ,


-- ** SignalFlags
    signalQueryReadSignalFlags              ,


-- ** SignalId
    signalQueryReadSignalId                 ,


-- ** SignalName
    signalQueryReadSignalName               ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.GObject.Types
import GI.GObject.Callbacks

newtype SignalQuery = SignalQuery (ForeignPtr SignalQuery)
-- | Construct a `SignalQuery` struct initialized to zero.
newZeroSignalQuery :: MonadIO m => m SignalQuery
newZeroSignalQuery = liftIO $ callocBytes 56 >>= wrapPtr SignalQuery

noSignalQuery :: Maybe SignalQuery
noSignalQuery = Nothing

signalQueryReadSignalId :: SignalQuery -> IO Word32
signalQueryReadSignalId s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO Word32
    return val

signalQueryReadSignalName :: SignalQuery -> IO T.Text
signalQueryReadSignalName s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO CString
    val' <- cstringToText val
    return val'

signalQueryReadItype :: SignalQuery -> IO GType
signalQueryReadItype s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO CGType
    let val' = GType val
    return val'

signalQueryReadSignalFlags :: SignalQuery -> IO [SignalFlags]
signalQueryReadSignalFlags s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 24) :: IO CUInt
    let val' = wordToGFlags val
    return val'

signalQueryReadReturnType :: SignalQuery -> IO GType
signalQueryReadReturnType s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 32) :: IO CGType
    let val' = GType val
    return val'

signalQueryReadNParams :: SignalQuery -> IO Word32
signalQueryReadNParams s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 40) :: IO Word32
    return val

-- XXX Skipped getter for "SignalQuery:param_types" :: Not implemented: "Don't know how to unpack C array of type TCArray False (-1) 5 (TBasicType TGType)"

type family ResolveSignalQueryMethod (t :: Symbol) (o :: *) :: * where
    ResolveSignalQueryMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveSignalQueryMethod t SignalQuery, MethodInfo info SignalQuery p) => IsLabelProxy t (SignalQuery -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveSignalQueryMethod t SignalQuery, MethodInfo info SignalQuery p) => IsLabel t (SignalQuery -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


